"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var index_exports = {};
__export(index_exports, {
  modulePathResolverPlugin: () => modulePathResolverPlugin,
  resolveModulePaths: () => resolveModulePaths
});
module.exports = __toCommonJS(index_exports);

// src/logger.ts
var Logger = class {
  #debug;
  constructor(debug = false) {
    this.#debug = debug;
  }
  log(message, color = "\x1B[30m%s\x1B[0m") {
    if (!this.#debug) {
      return;
    }
    console.log(color, message);
  }
  red(message) {
    this.log(message, "\x1B[31m%s\x1B[0m");
  }
  green(message) {
    this.log(message, "\x1B[32m%s\x1B[0m");
  }
  yellow(message) {
    this.log(message, "\x1B[33m%s\x1B[0m");
  }
  blue(message) {
    this.log(message, "\x1B[34m%s\x1B[0m");
  }
  magenta(message) {
    this.log(message, "\x1B[35m%s\x1B[0m");
  }
  cyan(message) {
    this.log(message, "\x1B[36m%s\x1B[0m");
  }
};

// src/utilities.ts
var import_fs = __toESM(require("fs"), 1);
var import_path = __toESM(require("path"), 1);
function createOutDir(outDir) {
  if (outDir !== "./" && !import_fs.default.existsSync(outDir)) {
    import_fs.default.mkdirSync(outDir, { recursive: true });
  }
}
function saveFile(outDir, fileName, contents) {
  const outputPath = import_path.default.join(outDir, fileName);
  import_fs.default.writeFileSync(outputPath, contents);
  return outputPath;
}

// src/module-path-resolver.ts
var log;
var userOptions = {
  outdir: "./",
  fileName: "custom-elements.json"
};
function resolveModulePaths(cem, options = {}) {
  log = new Logger(options.debug);
  if (options.skip) {
    log.yellow("[module-path-resolver] - Skipped");
    return;
  }
  updateOptions(options);
  log.log("[module-path-resolver] - Updating Custom Elements Manifest...");
  updateCemModules(cem);
  if (!options.usedByPlugin) {
    createOutDir(userOptions.outdir);
    saveFile(
      userOptions.outdir,
      userOptions.fileName,
      JSON.stringify(cem, null, 2)
    );
  }
  log.green("[module-path-resolver] - Custom Elements Manifest updated.");
}
function updateOptions(options = {}) {
  userOptions = { ...userOptions, ...options };
}
function updateCemModules(cem) {
  cem.modules.forEach((module2) => {
    const ogPath = module2.path;
    const ces = module2.declarations?.filter(
      (d) => d.customElement
    );
    if (ces.length) {
      ces.forEach((ce) => {
        if (userOptions.modulePathTemplate) {
          module2.path = userOptions.modulePathTemplate(
            ogPath,
            ce.name,
            ce.tagName
          );
          log.log(
            `[module-path-resolver] - Updated ${ce.name}'s module path: ${module2.path}`
          );
        }
        if (userOptions.typeDefinitionPathTemplate) {
          const typeDefPath = userOptions.typeDefinitionPathTemplate(
            ogPath,
            ce.name,
            ce.tagName
          );
          module2["typeDefinitionPath"] = typeDefPath;
          log.log(
            `[module-path-resolver] - Added type definition path to ${ce.name}: ${typeDefPath}`
          );
        }
        if (userOptions.definitionPathTemplate) {
          const exportPath = userOptions.definitionPathTemplate(
            ogPath,
            ce.name,
            ce.tagName
          );
          cem.modules.push({
            kind: "javascript-module",
            path: exportPath,
            deprecated: ce.deprecated || false,
            declarations: [],
            exports: [
              {
                kind: "custom-element-definition",
                name: ce.tagName || ce.name,
                declaration: {
                  name: ce.name,
                  module: module2.path
                }
              }
            ]
          });
          log.log(
            `[module-path-resolver] - Added definition export for ${ce.name}: ${exportPath}`
          );
        }
      });
    }
  });
}

// src/cem-plugin.ts
function modulePathResolverPlugin(options) {
  return {
    name: "@wc-toolkit/module-path-resolver",
    packageLinkPhase({ customElementsManifest }) {
      options.usedByPlugin = true;
      resolveModulePaths(customElementsManifest, options);
    }
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  modulePathResolverPlugin,
  resolveModulePaths
});
